unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, TypInfo, DTween, DTweenEasing;

type
  TViewIndex = 0..4;

  TPanel = class(ExtCtrls.TPanel)
  private
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
  protected
    procedure CMMouseenter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseleave(var Message: TMessage); message CM_MOUSELEAVE;
  public
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  TForm1 = class(TForm)
    btnSlideLeft: TButton;
    btnSlideRight: TButton;
    btnPlay: TButton;
    btnPause: TButton;
    btnBreak: TButton;
    pbGraph: TPaintBox;
    imgDragMe: TImage;
    pnColor1: TPanel;
    btnRingCenter: TButton;
    cbEasing1: TComboBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    pnColor2: TPanel;
    pnColor3: TPanel;
    pnColor4: TPanel;
    shpCircle: TShape;
    ListBox1: TListBox;
    cbEasing2: TComboBox;
    tbDivision: TTrackBar;
    lblDivision: TLabel;
    pnDragBase: TPanel;
    shpSquare1: TShape;
    shpSquare2: TShape;
    procedure btnPlayClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnRingCenterClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pbGraphPaint(Sender: TObject);
    procedure pnColorMouseEnter(Sender: TObject);
    procedure pnColorMouseLeave(Sender: TObject);
    procedure btnBreakClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure imgDragMeMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgDragMeMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure imgDragMeMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnSlideLeftClick(Sender: TObject);
    procedure btnSlideRightClick(Sender: TObject);
    procedure RadioButtonClick(Sender: TObject);
    procedure cbEasing1Select(Sender: TObject);
    procedure cbEasing2Select(Sender: TObject);
    procedure tbDivisionChange(Sender: TObject);
  private
    { Private 錾 }
    FDragging: Boolean;
    FPrevX: Integer;
    FPrevY: Integer;
    FDragOrgX: Integer;
    FDragOrgY: Integer;
    FGraphBuffer: TBitmap;
    FRingButtons: array[0..7] of TButton;
    FDragTarget: TPanel;
    FDragPanels: array[0..3] of TPanel;
    FGraphPrevX: Integer;
    FGraphPrevY: Integer;
    FViewIndex: TViewIndex;
    procedure RingButtonClick(Sender: TObject);
    procedure DragPanelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DragPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure DragPanelMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DrawEasingGraph(const Animation: Boolean = True);
    procedure RingTweenComplete(Sender: ITween);
    procedure ListTweenComplete(Sender: ITween);
    procedure GraphTweenUpdate(Sender: ITween);
    procedure SlideTweenComplete(Sender: ITween);
  public
    { Public 錾 }
    tween: ITween;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TPanel.CMMouseenter(var Message: TMessage);
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TPanel.CMMouseleave(var Message: TMessage);
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  size: TSize;
  i: Integer;
begin
  DoubleBuffered := True;

  imgDragMe.Canvas.Brush.Color := clFuchsia;
  imgDragMe.Canvas.FillRect(imgDragMe.ClientRect);
  imgDragMe.Canvas.Pen.Color := clBlack;
  imgDragMe.Canvas.Pen.Width := 2;
  imgDragMe.Canvas.Brush.Color := clWhite;
  imgDragMe.Canvas.Ellipse(imgDragMe.ClientRect);
  size := imgDragMe.Canvas.TextExtent('Drag me');
  imgDragMe.Canvas.TextOut((imgDragMe.Width-size.cx) div 2, (imgDragMe.Height-size.cy) div 2, 'Drag me');
  FDragOrgX := imgDragMe.Left;
  FDragOrgY := imgDragMe.Top;

  for i := Low(FRingButtons) to High(FRingButtons) do
  begin
    FRingButtons[i] := TButton.Create(Self);
    FRingButtons[i].Parent := btnRingCenter.Parent;
    FRingButtons[i].Left := btnRingCenter.Left + (btnRingCenter.Width - FRingButtons[i].Width) div 2;
    FRingButtons[i].Top := btnRingCenter.Top + (btnRingCenter.Height - FRingButtons[i].Height) div 2;
    FRingButtons[i].Width := 25;
    FRingButtons[i].Height := 25;
    FRingButtons[i].OnClick := RingButtonClick;
    FRingButtons[i].Visible := False;
  end;

  for i := Low(FDragPanels) to High(FDragPanels) do
  begin
    FDragPanels[i] := TPanel.Create(Self);
    FDragPanels[i].Parent := pnDragBase;
    FDragPanels[i].Color := clBtnFace;
    FDragPanels[i].Left := 2 + (pnDragBase.ClientWidth-4) div 4 * i;
    FDragPanels[i].Top := 2;
    FDragPanels[i].Width := (pnDragBase.ClientWidth-4) div 4;
    FDragPanels[i].Height := pnDragBase.ClientHeight-4;
    FDragPanels[i].Caption := 'Drag me';
    FDragPanels[i].Tag := i;
    FDragPanels[i].OnMouseDown := DragPanelMouseDown;
    FDragPanels[i].OnMouseMove := DragPanelMouseMove;
    FDragPanels[i].OnMouseUp := DragPanelMouseUp;
  end;

  for i := 1 to 4 do
  begin
    with TPanel(FindComponent('pnColor' + IntToStr(i))) do
    begin
      OnMouseEnter := pnColorMouseEnter;
      OnMouseLeave := pnColorMouseLeave;
    end;
  end;

  FViewIndex := 0;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  TTween.Abort;
  FGraphBuffer.Free;
end;

{ gDC[̍Đƈꎞ~̃f }

procedure TForm1.btnPlayClick(Sender: TObject);
begin
  if tween = nil then
  begin
    tween := TTween.Parallel([
      TTween.Cycle(2,
        TTween.Serial([
          TTween.TweenTo([shpCircle, shpSquare1, shpSquare2], ['$Left', 129], 1000, TEasing.SineEaseInOut),
          TTween.TweenTo([shpCircle, shpSquare1, shpSquare2], ['$Left', -129], 1000, TEasing.SineEaseInOut)
        ])),
      TTween.TweenTo(shpCircle, ['Top', shpSquare2.Top], 4000, TEasing.Linear)
    ]);
    tween.StopOnComplete := False;
  end;
  tween.Play;
end;

procedure TForm1.btnPauseClick(Sender: TObject);
begin
  if tween <> nil then
    tween.Pause;
end;

{ OR}hǂӂƏoďf }

procedure TForm1.btnRingCenterClick(Sender: TObject);
const
  D = 80;
  DirX: array[0..7] of Integer = (-1, 0, 1, 1, 1, 0, -1, -1);
  DirY: array[0..7] of Integer = (-1, -1, -1, 0, 1, 1, 1, 0);
var
  i: Integer;
  x, y: Integer;
  //t: ITween;
begin
  btnRingCenter.Enabled := False;
  for i := Low(FRingButtons) to High(FRingButtons) do
  begin
    FRingButtons[i].Left := btnRingCenter.Left + (btnRingCenter.Width - FRingButtons[i].Width) div 2;
    FRingButtons[i].Top := btnRingCenter.Top + (btnRingCenter.Height - FRingButtons[i].Height) div 2;
    FRingButtons[i].Enabled := True;
    FRingButtons[i].Visible := True;
    if Abs(DirX[i]) - Abs(DirY[i]) = 0 then
    begin
      x := Trunc(FRingButtons[i].Left + DirX[i] * D * 0.7);
      y := Trunc(FRingButtons[i].Top + DirY[i] * D * 0.7);
    end
    else begin
      x := FRingButtons[i].Left + DirX[i] * D;
      y := FRingButtons[i].Top + DirY[i] * D;
    end;

    // PTweenŎꍇ
    TTween.Tween(FRingButtons[i], [], ['Left', x, 'Top', y], 500, TEasing.BackEaseOut).Play;
    {
    // ʂTweenŎꍇ
    // Oɐݒ肵̂邽߁AOverwrite:=FalseK{)
    TTween.Tween(FRingButtons[i], ['Left', FRingButtons[i].Left], ['Left', x], 500, TEasing.BackEaseOut).Play;
    t := TTween.Tween(FRingButtons[i], ['Top', FRingButtons[i].Top], ['Top', y], 500, TEasing.BackEaseOut);
    t.Overwrite := False;
    t.Play;

    // ParalellŎꍇ
    // ParallelłOverwriteɂ閳Ȃ߁AOverwrite:=False͕sv
    TTween.Parallel([
      TTween.Tween(FRingButtons[i], ['Left', FRingButtons[i].Left], ['Left', x], 500, TEasing.BackEaseOut),
      TTween.Tween(FRingButtons[i], ['Top', FRingButtons[i].Top], ['Top', y], 500, TEasing.BackEaseOut)
    ]).Play;
    }
  end;
end;

procedure TForm1.RingButtonClick(Sender: TObject);
var
  i: Integer;
  x, y: Integer;
  tweens: array[0..7] of ITween;
  t: ITween;
begin
  for i := Low(FRingButtons) to High(FRingButtons) do
  begin
    FRingButtons[i].Enabled := False;
    x := btnRingCenter.Left + (btnRingCenter.Width - FRingButtons[i].Width) div 2;
    y := btnRingCenter.Top + (btnRingCenter.Height - FRingButtons[i].Height) div 2;
    tweens[i] := TTween.Parallel([
      TTween.TweenTo(FRingButtons[i], ['Left', x], 500, TEasing.BackEaseIn),
      TTween.TweenTo(FRingButtons[i], ['Top', y], 500, TEasing.BackEaseIn)
    ]);
  end;
  t := TTween.Parallel(tweens);
  t.OnComplete := RingTweenComplete;
  t.Play;
end;

procedure TForm1.RingTweenComplete(Sender: ITween);
var
  i: Integer;
begin
  for i := Low(FRingButtons) to High(FRingButtons) do
    FRingButtons[i].Visible := False;
  btnRingCenter.Enabled := True
end;

{ hbOČɖ߂f }

procedure TForm1.imgDragMeMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then Exit;
  TTween.Abort(Sender);
  FDragging := True;
  FPrevX := X;
  FPrevY := Y;
end;

procedure TForm1.imgDragMeMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  pt: TPoint;
begin
  if not FDragging then Exit;

  pt := imgDragMe.Parent.ScreenToClient(imgDragMe.ClientToScreen(Point(X, Y)));
  imgDragMe.Left := pt.X-FPrevX;
  imgDragMe.Top := pt.Y-FPrevY;
end;

procedure TForm1.imgDragMeMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if not FDragging then Exit;
  FDragging := False;

  TTween.TweenTo([Sender], ['Left', FDragOrgX, 'Top', FDragOrgY], 1000, TEasing.ElasticEaseOut).Play;
end;

{ Xg{bNX\ɂAgւčēx\܂
  Xgɂ̓TvƂăgDC[\ȃvpeB񋓂܂ }

procedure TForm1.RadioButtonClick(Sender: TObject);
var
  t1, t2: ITween;
begin
  t1 := TTween.TweenTo(ListBox1, ['Height', 0], 500, TEasing.BackEaseIn);
  t1.OnComplete := ListTweenComplete;
  t2 := TTween.Delay(200, 0,
    TTween.Tween(ListBox1, ['Height', 0], ['Height', 370], 500, TEasing.BackEaseOut));
  TTween.Serial([t1, t2]).Play;
end;

procedure TForm1.ListTweenComplete(Sender: ITween);
var
  i, j, viewIdx: Integer;
  cnt: Integer;
  propList: PPropList;
  propCount: Integer;
  radioButton: TRadioButton;
begin
  ListBox1.Clear;
  ListBox1.Items.Add('// gDC[\ȃvpeBꗗ');
  if RadioButton1.Checked then
    radioButton := RadioButton1
  else if RadioButton2.Checked then
    radioButton := RadioButton2
  else if RadioButton3.Checked then
    radioButton := RadioButton3
  else
    radioButton := RadioButton4;
  viewIdx := StrToInt(radioButton.Name[Length(radioButton.Name)])-1;
  for i := 0 to ControlCount-1 do
  begin
    if Controls[i].Tag <> viewIdx then Continue;
    propCount := GetPropList(PTypeInfo(Controls[i].ClassInfo), propList);
    cnt := 0;
    if propCount > 0 then
    try
      ListBox1.Items.Add(Format('%s = class(%s)', [Controls[i].ClassName, Controls[i].ClassParent.ClassName]));
      for j := 0 to propCount-1 do
      begin
        if propList[j] = nil then Continue;
        if not (propList[j].PropType^^.Kind in [tkInteger, tkFloat]) then Continue;
        if (propList[j].GetProc = nil) or (propList[j].SetProc = nil) then Continue;
        ListBox1.Items.Add(Format('  property %s: %s;', [propList[j].Name, propList[j].PropType^^.Name]));
        Inc(cnt);
      end;
    finally
      FreeMem(propList);
    end;

    if cnt = 0 then
    begin
      ListBox1.Items[ListBox1.Items.Count-1] := ListBox1.Items[ListBox1.Items.Count-1] + ';';
    end
    else begin
      ListBox1.Items.Add('end;');
    end;
  end;
end;

{ Fωf }

const
  PanelColors: array[0..3] of TColor = (clAqua, clFuchsia, clYellow, clBlack);
procedure TForm1.pnColorMouseEnter(Sender: TObject);
var
  cl: TColor;
begin
  cl := PanelColors[StrToInt(TPanel(Sender).Name[8])-1];
  TTween.TweenTo([Sender], ['Color', cl], 200, TEasing.SineEaseIn).Play;
end;

procedure TForm1.pnColorMouseLeave(Sender: TObject);
begin
  TTween.TweenTo([Sender], ['Color', clBtnFace], 1000, TEasing.SineEaseOut).Play;
end;

{ C[WO֐OtƂĕ\f }

function CustomEasing(const t, b, c, d: Double): Double;
begin
  if t / d < 0.5 then
    Result := b
  else
    Result := b + c;
end;

function GetEasingByIndex(Index: Integer): IEasing;
begin
  case Index of
    1: Result := TEasing.Linear;
    2: Result := TEasing.SineEaseIn;
    3: Result := TEasing.SineEaseOut;
    4: Result := TEasing.SineEaseInOut;
    5: Result := TEasing.SineEaseOutIn;
    6: Result := TEasing.QuadEaseIn;
    7: Result := TEasing.QuadEaseOut;
    8: Result := TEasing.QuadEaseInOut;
    9: Result := TEasing.QuadEaseOutIn;
    10: Result := TEasing.CircEaseIn;
    11: Result := TEasing.CircEaseOut;
    12: Result := TEasing.CircEaseInOut;
    13: Result := TEasing.CircEaseOutIn;
    14: Result := TEasing.CubicEaseIn;
    15: Result := TEasing.CubicEaseOut;
    16: Result := TEasing.CubicEaseInOut;
    17: Result := TEasing.CubicEaseOutIn;
    18: Result := TEasing.ExpoEaseIn;
    19: Result := TEasing.ExpoEaseOut;
    20: Result := TEasing.ExpoEaseInOut;
    21: Result := TEasing.ExpoEaseOutIn;
    22: Result := TEasing.QuartEaseIn;
    23: Result := TEasing.QuartEaseOut;
    24: Result := TEasing.QuartEaseInOut;
    25: Result := TEasing.QuartEaseOutIn;
    26: Result := TEasing.QuintEaseIn;
    27: Result := TEasing.QuintEaseOut;
    28: Result := TEasing.QuintEaseInOut;
    29: Result := TEasing.QuintEaseOutIn;
    30: Result := TEasing.BackEaseIn;
    31: Result := TEasing.BackEaseOut;
    32: Result := TEasing.BackEaseInOut;
    33: Result := TEasing.BackEaseOutIn;
    34: Result := TEasing.BounceEaseIn;
    35: Result := TEasing.BounceEaseOut;
    36: Result := TEasing.BounceEaseInOut;
    37: Result := TEasing.BounceEaseOutIn;
    38: Result := TEasing.ElasticEaseIn;
    39: Result := TEasing.ElasticEaseOut;
    40: Result := TEasing.ElasticEaseInOut;
    41: Result := TEasing.ElasticEaseOutIn;
    42: Result := TEasing.Construct(CustomEasing);
    else Result := nil;
  end;
end;

procedure TForm1.cbEasing1Select(Sender: TObject);
begin
  cbEasing2.Visible := cbEasing1.ItemIndex >= 1;
  tbDivision.Visible := cbEasing2.Visible and (cbEasing2.ItemIndex >= 1);
  lblDivision.Visible := cbEasing2.Visible and (cbEasing2.ItemIndex >= 1);
  DrawEasingGraph;
end;

procedure TForm1.cbEasing2Select(Sender: TObject);
begin
  tbDivision.Visible := cbEasing2.ItemIndex >= 1;
  lblDivision.Visible := cbEasing2.ItemIndex >= 1;
  DrawEasingGraph;
end;

procedure TForm1.tbDivisionChange(Sender: TObject);
begin
  lblDivision.Caption := Format('%.2f', [tbDivision.Position / 100]);
  DrawEasingGraph(False);
end;

procedure TForm1.DrawEasingGraph(const Animation: Boolean = True);
var
  t: ITween;
  easing, easing1, easing2: IEasing;
  padX: Integer;
  padY: Integer;
  i, x, y: Integer;
begin
  if FGraphBuffer = nil then
  begin
    FGraphBuffer := TBitmap.Create;
    FGraphBuffer.Width := pbGraph.Width;
    FGraphBuffer.Height := pbGraph.Height;
  end;
  padX := (FGraphBuffer.Width - 200) div 2;
  padY := (FGraphBuffer.Height - 200) div 2;
  FGraphBuffer.Canvas.Brush.Color := clWhite;
  FGraphBuffer.Canvas.FillRect(Rect(0, 0, FGraphBuffer.Width, FGraphBuffer.Height));
  FGraphBuffer.Canvas.Pen.Color := clBlack;
  FGraphBuffer.Canvas.Pen.Width := 1;
  FGraphBuffer.Canvas.Rectangle(Bounds(padX, padY, 200, 200));

  easing1 := GetEasingByIndex(cbEasing1.ItemIndex);
  if not Assigned(easing1) then Exit;
  easing2 := GetEasingByIndex(cbEasing2.ItemIndex);
  if not Assigned(easing2) then
    easing := easing1
  else
    easing := TEasing.Combine(easing1, easing2, tbDivision.Position / 100);

  FGraphPrevX := padX;
  FGraphPrevY := 200+padY;
  if Animation then
  begin
    t := TTween.Tween(Form1, ['Tag', 200+padY], ['Tag', padY], 1000, easing);
    t.OnUpdate := GraphTweenUpdate;
    t.Play;
  end
  else begin
    FGraphBuffer.Canvas.Pen.Color := clSkyBlue;
    FGraphBuffer.Canvas.Pen.Width := 2;
    FGraphBuffer.Canvas.MoveTo(padX, padX+200);
    for i := 1 to 200 do
    begin
      x := padX + i;
      y := Round(easing.Calculate(i, padY+200, -200, 200));
      FGraphBuffer.Canvas.LineTo(x, y);
    end;
    pbGraph.Invalidate;
  end
end;

procedure TForm1.GraphTweenUpdate(Sender: ITween);
var
  x, y: Integer;
begin
  FGraphBuffer.Canvas.Pen.Color := clSkyBlue;
  FGraphBuffer.Canvas.Pen.Width := 2;
  FGraphBuffer.Canvas.MoveTo(FGraphPrevX, FGraphPrevY);
  x := Round((Sender.Position / Sender.Duration) * 200 + (FGraphBuffer.Width - 200) div 2);
  y := Form1.Tag;
  FGraphBuffer.Canvas.LineTo(x, y);
  FGraphPrevX := x;
  FGraphPrevY := y;
  pbGraph.Invalidate;
end;

procedure TForm1.pbGraphPaint(Sender: TObject);
begin
  if FGraphBuffer <> nil then
    pbGraph.Canvas.Draw(0, 0, FGraphBuffer);
end;

{ plӂƓւĂ݂f }

procedure TForm1.DragPanelMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  r: TRect;
begin
  if Button <> mbLeft then Exit;
  TTween.Abort(Sender);
  FDragging := True;
  FPrevX := X;
  FPrevY := Y;
  FDragTarget := TPanel(Sender);
  FDragTarget.BringToFront;
  TTween.TweenTo(FDragTarget, ['Color', clBtnHighlight], 200).Play;

  r.Left := X+2;
  r.Top := 2;
  r.Right := pnDragBase.ClientWidth-(FDragTarget.Width-X)-1;
  r.Bottom := pnDragBase.ClientHeight-4;
  r.TopLeft := pnDragBase.ClientToScreen(r.TopLeft);
  r.BottomRight := pnDragBase.ClientToScreen(r.BottomRight);
  ClipCursor(@r);
end;

procedure TForm1.DragPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  pt: TPoint;
  i, idxNew, idxMin, idxMax, dir: Integer;
begin
  if not FDragging then Exit;
  if Sender <> FDragTarget then Exit;

  pt := FDragTarget.Parent.ScreenToClient(FDragTarget.ClientToScreen(Point(X, Y)));
  FDragTarget.Left := pt.X-FPrevX;

  // ړΏۂƂȂpl͈̔͂ƕ
  idxNew := (FDragTarget.Left + FDragTarget.Width div 2) div FDragTarget.Width;
  if idxNew > FDragTarget.Tag then
  begin
    idxMin := FDragTarget.Tag;
    idxMax := idxNew;
    dir := -1;
  end
  else if idxNew < FDragTarget.Tag then
  begin
    idxMin := idxNew;
    idxMax := FDragTarget.Tag;
    dir := 1;
  end
  else begin
    Exit;
  end;

  // ړΏۂ̃plړ
  for i := 0 to pnDragBase.ControlCount-1 do
  begin
    if pnDragBase.Controls[i] = FDragTarget then Continue;
    if (pnDragBase.Controls[i].Tag >= idxMin) and (pnDragBase.Controls[i].Tag <= idxMax) then
    begin
      pnDragBase.Controls[i].Tag := pnDragBase.Controls[i].Tag + dir;
      TTween.TweenTo(pnDragBase.Controls[i], ['Left', pnDragBase.Controls[i].Tag * pnDragBase.Controls[i].Width + 2], 500, TEasing.CubicEaseOut).Play;
    end;
  end;
  FDragTarget.Tag := idxNew;
end;

procedure TForm1.DragPanelMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if not FDragging then Exit;
  if Sender <> FDragTarget then Exit;
  FDragging := False;
  ClipCursor(nil);

  // ͂łpl𐳂ʒuɈړ
  TTween.Parallel([
    TTween.TweenTo(FDragTarget, ['Left', FDragTarget.Tag * FDragTarget.Width + 2], 500, TEasing.CubicEaseOut),
    TTween.TweenTo(FDragTarget, ['Color', clBtnFace], 200)
  ]).Play;
end;

{ Rg[Ă݂f }

procedure TForm1.btnBreakClick(Sender: TObject);
var
  i, y: Integer;
  t1, t2, t3, t: ITween;
begin
  with TButton(Sender).Parent do
    for i := 0 to ControlCount-1 do
    begin
      y := ClientHeight-Controls[i].Height;
      t1 := TTween.Delay(0, 500, TTween.TweenTo(Controls[i], ['Top', y], 1000, TEasing.BounceEaseOut));
      t2 := TTween.Tween(Controls[i], ['Top', y], ['Top', ClientHeight], 200, TEasing.SineEaseOut);
      t3 := TTween.Tween(Controls[i], ['Top', -Controls[i].Height], ['Top', Controls[i].Top], 800, TEasing.QuadEaseOut);
      t := TTween.Serial([t1, t2, t3]);
      t.Play;
    end;
end;

{ ʂXChĂ݂f }

procedure TForm1.btnSlideLeftClick(Sender: TObject);
var
  i: Integer;
  t: ITween;
begin
  if FViewIndex <= Low(TViewIndex) then Exit;
  Dec(FViewIndex);
  btnSlideLeft.Enabled := False;
  btnSlideRight.Enabled := False;
  btnBreak.Enabled := False;

  for i := 0 to ControlCount-1 do
  begin
    if Controls[i].Tag = -1 then Continue;
    t := TTween.TweenTo(Controls[i], ['$Left', ClientWidth], 500, TEasing.BackEaseOut);
    t.Play;
  end;
  if t <> nil then
    t.OnComplete := SlideTweenComplete;
end;

procedure TForm1.btnSlideRightClick(Sender: TObject);
var
  i: Integer;
  t: ITween;
begin
  if FViewIndex >= High(TViewIndex) then Exit;
  Inc(FViewIndex);
  btnSlideLeft.Enabled := False;
  btnSlideRight.Enabled := False;
  btnBreak.Enabled := False;

  for i := 0 to ControlCount-1 do
  begin
    if Controls[i].Tag = -1 then Continue;
    t := TTween.TweenTo(Controls[i], ['$Left', -ClientWidth], 500, TEasing.BackEaseOut);
    t.Play;
  end;
  if t <> nil then
    t.OnComplete := SlideTweenComplete;
end;

procedure TForm1.SlideTweenComplete(Sender: ITween);
begin
  btnSlideLeft.Enabled := FViewIndex > Low(TViewIndex);
  btnSlideRight.Enabled := FViewIndex < High(TViewIndex);
  btnBreak.Enabled := True;
end;

end.
