unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects3D, FMX.Types3D,
  FMX.Layers3D, DTween, DTween.Easing, Math;

type
  TForm1 = class(TForm3D)
    Light1: TLight;
    Camera1: TCamera;
    Grid3D1: TGrid3D;
    Layer3D1: TLayer3D;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Form3DCreate(Sender: TObject);
  private
    { private 錾 }
    FRot: Single;
    FSpheres: array[0..19] of TSphere;
    procedure SetRot(const Value: Single);
  public
    { public 錾 }
    property Rot: Single read FRot write SetRot;
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

procedure TForm1.Form3DCreate(Sender: TObject);
begin
  Randomize;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i: Integer;
  t: ITween;
begin
  Layer3D1.Visible := False;

  t := TTween.Parallel([
    TTween.Tween(Camera1.Position, ['Y', 5], ['Y', -5], 6000, TEasing.CubicEaseInOut),
    TTween.Tween(Self, ['Rot', 0], ['Rot', 90], 7000, TEasing.CubicEaseInOut)
  ]);

  for i := Low(FSpheres) to High(FSpheres) do
  begin
    if FSpheres[i] = nil then
    begin
      FSpheres[i] := TSphere.Create(Self);
      FSpheres[i].Parent := Self;
      FSpheres[i].Position.X := RandomRange(-50, 50);
      FSpheres[i].Position.Y := -100;
      FSpheres[i].Position.Z := RandomRange(50-20, 50+50);
      FSpheres[i].RotationAngle.X := 50;
      FSpheres[i].RotationAngle.Y := 50;
      FSpheres[i].RotationAngle.Z := 50;
      FSpheres[i].Material.Diffuse := claSkyblue;
      FSpheres[i].Material.Emissive := claMidnightblue;
      FSpheres[i].SetSize(5, 5, 5);
    end;

    t := TTween.Parallel([
      t,
      TTween.Tween(FSpheres[i].Position, ['Y', -100], ['Y', 5], 3000+Random(2000), TEasing.BounceEaseOut)
    ]);
  end;

  t := TTween.Serial([
    TTween.Scale(0.5, TTween.Slice(0.0, 0.45, t, True)),
    TTween.Scale(15, TTween.Slice(0.45, 0.5, t, True)),
    TTween.Scale(0.5, TTween.Slice(0.5, 1.0, t, True))
  ]);

  t := TTween.Serial([t, TTween.Reverse(t)]);
  t.StopOnComplete := False;
  t.Play;
end;

procedure TForm1.SetRot(const Value: Single);
begin
  FRot := Value;
  Camera1.Position.X := -25 * Sin(DegToRad(FRot));
  Camera1.Position.Z := -25 * Cos(DegToRad(FRot))+25;
  Camera1.ResetRotationAngle;
  Camera1.RotationAngle.Y := FRot / 3;
  Camera1.RotationAngle.X := -FRot / 3 + 10;
end;

end.
